@@ -20,18 +20,19 @@ import (
 	"sync"
 	"time"
 
-	"github.com/cenkalti/backoff"
-	"github.com/cespare/xxhash"
-	"github.com/go-kit/kit/log"
-	"github.com/go-kit/kit/log/level"
+	"github.com/cenkalti/backoff/v4"
+	"github.com/cespare/xxhash/v2"
+	"github.com/go-kit/log"
+	"github.com/go-kit/log/level"
+	"github.com/pkg/errors"
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/common/model"
 
-	"github.com/prometheus/alertmanager/cluster"
 	"github.com/prometheus/alertmanager/inhibit"
 	"github.com/prometheus/alertmanager/nflog"
 	"github.com/prometheus/alertmanager/nflog/nflogpb"
 	"github.com/prometheus/alertmanager/silence"
+	"github.com/prometheus/alertmanager/timeinterval"
 	"github.com/prometheus/alertmanager/types"
 )
 
@@ -40,6 +41,12 @@ type ResolvedSender interface {
 	SendResolved() bool
 }
 
+// Peer represents the cluster node from where we are the sending the notification.
+type Peer interface {
+	// WaitReady waits until the node silences and notifications have settled before attempting to send a notification.
+	WaitReady(context.Context) error
+}
+
 // MinTimeout is the minimum timeout that is set for the context of a call
 // to a notification pipeline.
 const MinTimeout = 10 * time.Second
@@ -90,6 +97,11 @@ func (i *Integration) Index() int {
 	return i.idx
 }
 
+// String implements the Stringer interface.
+func (i *Integration) String() string {
+	return fmt.Sprintf("%s[%d]", i.name, i.idx)
+}
+
 // notifyKey defines a custom type with which a context is populated to
 // avoid accidental collisions.
 type notifyKey int
@@ -102,6 +114,8 @@ const (
 	keyFiringAlerts
 	keyResolvedAlerts
 	keyNow
+	keyMuteTimeIntervals
+	keyActiveTimeIntervals
 )
 
 // WithReceiverName populates a context with a receiver name.
@@ -139,6 +153,15 @@ func WithRepeatInterval(ctx context.Context, t time.Duration) context.Context {
 	return context.WithValue(ctx, keyRepeatInterval, t)
 }
 
+// WithMuteTimeIntervals populates a context with a slice of mute time names.
+func WithMuteTimeIntervals(ctx context.Context, mt []string) context.Context {
+	return context.WithValue(ctx, keyMuteTimeIntervals, mt)
+}
+
+func WithActiveTimeIntervals(ctx context.Context, at []string) context.Context {
+	return context.WithValue(ctx, keyActiveTimeIntervals, at)
+}
+
 // RepeatInterval extracts a repeat interval from the context. Iff none exists, the
 // second argument is false.
 func RepeatInterval(ctx context.Context) (time.Duration, bool) {
@@ -188,6 +211,20 @@ func ResolvedAlerts(ctx context.Context) ([]uint64, bool) {
 	return v, ok
 }
 
+// MuteTimeIntervalNames extracts a slice of mute time names from the context. If and only if none exists, the
+// second argument is false.
+func MuteTimeIntervalNames(ctx context.Context) ([]string, bool) {
+	v, ok := ctx.Value(keyMuteTimeIntervals).([]string)
+	return v, ok
+}
+
+// ActiveTimeIntervalNames extracts a slice of active time names from the context. If none exists, the
+// second argument is false.
+func ActiveTimeIntervalNames(ctx context.Context) ([]string, bool) {
+	v, ok := ctx.Value(keyActiveTimeIntervals).([]string)
+	return v, ok
+}
+
 // A Stage processes alerts under the constraints of the given context.
 type Stage interface {
 	Exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error)
@@ -206,24 +243,36 @@ type NotificationLog interface {
 	Query(params ...nflog.QueryParam) ([]*nflogpb.Entry, error)
 }
 
-type metrics struct {
-	numNotifications           *prometheus.CounterVec
-	numFailedNotifications     *prometheus.CounterVec
-	notificationLatencySeconds *prometheus.HistogramVec
+type Metrics struct {
+	numNotifications                   *prometheus.CounterVec
+	numTotalFailedNotifications        *prometheus.CounterVec
+	numNotificationRequestsTotal       *prometheus.CounterVec
+	numNotificationRequestsFailedTotal *prometheus.CounterVec
+	notificationLatencySeconds         *prometheus.HistogramVec
 }
 
-func newMetrics(r prometheus.Registerer) *metrics {
-	m := &metrics{
+func NewMetrics(r prometheus.Registerer) *Metrics {
+	m := &Metrics{
 		numNotifications: prometheus.NewCounterVec(prometheus.CounterOpts{
 			Namespace: "alertmanager",
 			Name:      "notifications_total",
 			Help:      "The total number of attempted notifications.",
 		}, []string{"integration"}),
-		numFailedNotifications: prometheus.NewCounterVec(prometheus.CounterOpts{
+		numTotalFailedNotifications: prometheus.NewCounterVec(prometheus.CounterOpts{
 			Namespace: "alertmanager",
 			Name:      "notifications_failed_total",
 			Help:      "The total number of failed notifications.",
 		}, []string{"integration"}),
+		numNotificationRequestsTotal: prometheus.NewCounterVec(prometheus.CounterOpts{
+			Namespace: "alertmanager",
+			Name:      "notification_requests_total",
+			Help:      "The total number of attempted notification requests.",
+		}, []string{"integration"}),
+		numNotificationRequestsFailedTotal: prometheus.NewCounterVec(prometheus.CounterOpts{
+			Namespace: "alertmanager",
+			Name:      "notification_requests_failed_total",
+			Help:      "The total number of failed notification requests.",
+		}, []string{"integration"}),
 		notificationLatencySeconds: prometheus.NewHistogramVec(prometheus.HistogramOpts{
 			Namespace: "alertmanager",
 			Name:      "notification_latency_seconds",
@@ -233,7 +282,6 @@ func newMetrics(r prometheus.Registerer) *metrics {
 	}
 	for _, integration := range []string{
 		"email",
-		"hipchat",
 		"pagerduty",
 		"wechat",
 		"pushover",
@@ -241,22 +289,30 @@ func newMetrics(r prometheus.Registerer) *metrics {
 		"opsgenie",
 		"webhook",
 		"victorops",
+		"sns",
+		"telegram",
 	} {
 		m.numNotifications.WithLabelValues(integration)
-		m.numFailedNotifications.WithLabelValues(integration)
+		m.numTotalFailedNotifications.WithLabelValues(integration)
+		m.numNotificationRequestsTotal.WithLabelValues(integration)
+		m.numNotificationRequestsFailedTotal.WithLabelValues(integration)
 		m.notificationLatencySeconds.WithLabelValues(integration)
 	}
-	r.MustRegister(m.numNotifications, m.numFailedNotifications, m.notificationLatencySeconds)
+	r.MustRegister(
+		m.numNotifications, m.numTotalFailedNotifications,
+		m.numNotificationRequestsTotal, m.numNotificationRequestsFailedTotal,
+		m.notificationLatencySeconds,
+	)
 	return m
 }
 
 type PipelineBuilder struct {
-	metrics *metrics
+	metrics *Metrics
 }
 
 func NewPipelineBuilder(r prometheus.Registerer) *PipelineBuilder {
 	return &PipelineBuilder{
-		metrics: newMetrics(r),
+		metrics: NewMetrics(r),
 	}
 }
 
@@ -266,18 +322,21 @@ func (pb *PipelineBuilder) New(
 	wait func() time.Duration,
 	inhibitor *inhibit.Inhibitor,
 	silencer *silence.Silencer,
+	times map[string][]timeinterval.TimeInterval,
 	notificationLog NotificationLog,
-	peer *cluster.Peer,
+	peer Peer,
 ) RoutingStage {
 	rs := make(RoutingStage, len(receivers))
 
 	ms := NewGossipSettleStage(peer)
 	is := NewMuteStage(inhibitor)
 	ss := NewMuteStage(silencer)
+	tms := NewTimeMuteStage(times)
+	tas := NewTimeActiveStage(times)
 
 	for name := range receivers {
 		st := createReceiverStage(name, receivers[name], wait, notificationLog, pb.metrics)
-		rs[name] = MultiStage{ms, is, ss, st}
+		rs[name] = MultiStage{ms, is, tas, tms, ss, st}
 	}
 	return rs
 }
@@ -288,7 +347,7 @@ func createReceiverStage(
 	integrations []Integration,
 	wait func() time.Duration,
 	notificationLog NotificationLog,
-	metrics *metrics,
+	metrics *Metrics,
 ) Stage {
 	var fs FanoutStage
 	for i := range integrations {
@@ -316,12 +375,12 @@ type RoutingStage map[string]Stage
 func (rs RoutingStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
 	receiver, ok := ReceiverName(ctx)
 	if !ok {
-		return ctx, nil, fmt.Errorf("receiver missing")
+		return ctx, nil, errors.New("receiver missing")
 	}
 
 	s, ok := rs[receiver]
 	if !ok {
-		return ctx, nil, fmt.Errorf("stage for receiver missing")
+		return ctx, nil, errors.New("stage for receiver missing")
 	}
 
 	return s.Exec(ctx, l, alerts...)
@@ -362,14 +421,6 @@ func (fs FanoutStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.A
 		go func(s Stage) {
 			if _, _, err := s.Exec(ctx, l, alerts...); err != nil {
 				me.Add(err)
-				lvl := level.Error(l)
-				if ctx.Err() == context.Canceled {
-					// It is expected for the context to be canceled on
-					// configuration reload or shutdown. In this case, the
-					// message should only be logged at the debug level.
-					lvl = level.Debug(l)
-				}
-				lvl.Log("msg", "Error on notify", "err", err, "context_err", ctx.Err())
 			}
 			wg.Done()
 		}(s)
@@ -384,17 +435,19 @@ func (fs FanoutStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.A
 
 // GossipSettleStage waits until the Gossip has settled to forward alerts.
 type GossipSettleStage struct {
-	peer *cluster.Peer
+	peer Peer
 }
 
 // NewGossipSettleStage returns a new GossipSettleStage.
-func NewGossipSettleStage(p *cluster.Peer) *GossipSettleStage {
+func NewGossipSettleStage(p Peer) *GossipSettleStage {
 	return &GossipSettleStage{peer: p}
 }
 
-func (n *GossipSettleStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
+func (n *GossipSettleStage) Exec(ctx context.Context, _ log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
 	if n.peer != nil {
-		n.peer.WaitReady()
+		if err := n.peer.WaitReady(ctx); err != nil {
+			return ctx, nil, err
+		}
 	}
 	return ctx, alerts, nil
 }
@@ -410,7 +463,7 @@ func NewMuteStage(m types.Muter) *MuteStage {
 }
 
 // Exec implements the Stage interface.
-func (n *MuteStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
+func (n *MuteStage) Exec(ctx context.Context, _ log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
 	var filtered []*types.Alert
 	for _, a := range alerts {
 		// TODO(fabxc): increment total alerts counter.
@@ -437,7 +490,7 @@ func NewWaitStage(wait func() time.Duration) *WaitStage {
 }
 
 // Exec implements the Stage interface.
-func (ws *WaitStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
+func (ws *WaitStage) Exec(ctx context.Context, _ log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
 	select {
 	case <-time.After(ws.wait()):
 	case <-ctx.Done():
@@ -472,27 +525,21 @@ func utcNow() time.Time {
 	return time.Now().UTC()
 }
 
-var hashBuffers = sync.Pool{}
-
-func getHashBuffer() []byte {
-	b := hashBuffers.Get()
-	if b == nil {
-		return make([]byte, 0, 1024)
-	}
-	return b.([]byte)
+// Wrap a slice in a struct so we can store a pointer in sync.Pool
+type hashBuffer struct {
+	buf []byte
 }
 
-func putHashBuffer(b []byte) {
-	b = b[:0]
-	//lint:ignore SA6002 relax staticcheck verification.
-	hashBuffers.Put(b)
+var hashBuffers = sync.Pool{
+	New: func() interface{} { return &hashBuffer{buf: make([]byte, 0, 1024)} },
 }
 
 func hashAlert(a *types.Alert) uint64 {
 	const sep = '\xff'
 
-	b := getHashBuffer()
-	defer putHashBuffer(b)
+	hb := hashBuffers.Get().(*hashBuffer)
+	defer hashBuffers.Put(hb)
+	b := hb.buf[:0]
 
 	names := make(model.LabelNames, 0, len(a.Labels))
 
@@ -544,15 +591,15 @@ func (n *DedupStage) needsUpdate(entry *nflogpb.Entry, firing, resolved map[uint
 }
 
 // Exec implements the Stage interface.
-func (n *DedupStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
+func (n *DedupStage) Exec(ctx context.Context, _ log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
 	gkey, ok := GroupKey(ctx)
 	if !ok {
-		return ctx, nil, fmt.Errorf("group key missing")
+		return ctx, nil, errors.New("group key missing")
 	}
 
 	repeatInterval, ok := RepeatInterval(ctx)
 	if !ok {
-		return ctx, nil, fmt.Errorf("repeat interval missing")
+		return ctx, nil, errors.New("repeat interval missing")
 	}
 
 	firingSet := map[uint64]struct{}{}
@@ -586,7 +633,7 @@ func (n *DedupStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Al
 	case 1:
 		entry = entries[0]
 	default:
-		return ctx, nil, fmt.Errorf("unexpected entry result size %d", len(entries))
+		return ctx, nil, errors.Errorf("unexpected entry result size %d", len(entries))
 	}
 
 	if n.needsUpdate(entry, firingSet, resolvedSet, repeatInterval) {
@@ -600,11 +647,11 @@ func (n *DedupStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Al
 type RetryStage struct {
 	integration Integration
 	groupName   string
-	metrics     *metrics
+	metrics     *Metrics
 }
 
 // NewRetryStage returns a new instance of a RetryStage.
-func NewRetryStage(i Integration, groupName string, metrics *metrics) *RetryStage {
+func NewRetryStage(i Integration, groupName string, metrics *Metrics) *RetryStage {
 	return &RetryStage{
 		integration: i,
 		groupName:   groupName,
@@ -612,8 +659,16 @@ func NewRetryStage(i Integration, groupName string, metrics *metrics) *RetryStag
 	}
 }
 
-// Exec implements the Stage interface.
 func (r RetryStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
+	r.metrics.numNotifications.WithLabelValues(r.integration.Name()).Inc()
+	ctx, alerts, err := r.exec(ctx, l, alerts...)
+	if err != nil {
+		r.metrics.numTotalFailedNotifications.WithLabelValues(r.integration.Name()).Inc()
+	}
+	return ctx, alerts, err
+}
+
+func (r RetryStage) exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
 	var sent []*types.Alert
 
 	// If we shouldn't send notifications for resolved alerts, but there are only
@@ -622,7 +677,7 @@ func (r RetryStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Ale
 	if !r.integration.SendResolved() {
 		firing, ok := FiringAlerts(ctx)
 		if !ok {
-			return ctx, nil, fmt.Errorf("firing alerts missing")
+			return ctx, nil, errors.New("firing alerts missing")
 		}
 		if len(firing) == 0 {
 			return ctx, alerts, nil
@@ -636,24 +691,28 @@ func (r RetryStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Ale
 		sent = alerts
 	}
 
+	b := backoff.NewExponentialBackOff()
+	b.MaxElapsedTime = 0 // Always retry.
+
+	tick := backoff.NewTicker(b)
+	defer tick.Stop()
+
 	var (
 		i    = 0
-		b    = backoff.NewExponentialBackOff()
-		tick = backoff.NewTicker(b)
 		iErr error
 	)
-	defer tick.Stop()
+	l = log.With(l, "receiver", r.groupName, "integration", r.integration.String())
 
 	for {
 		i++
 		// Always check the context first to not notify again.
 		select {
 		case <-ctx.Done():
-			if iErr != nil {
-				return ctx, nil, iErr
+			if iErr == nil {
+				iErr = ctx.Err()
 			}
 
-			return ctx, nil, ctx.Err()
+			return ctx, nil, errors.Wrapf(iErr, "%s/%s: notify retry canceled after %d attempts", r.groupName, r.integration.String(), i)
 		default:
 		}
 
@@ -662,26 +721,34 @@ func (r RetryStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Ale
 			now := time.Now()
 			retry, err := r.integration.Notify(ctx, sent...)
 			r.metrics.notificationLatencySeconds.WithLabelValues(r.integration.Name()).Observe(time.Since(now).Seconds())
-			r.metrics.numNotifications.WithLabelValues(r.integration.Name()).Inc()
+			r.metrics.numNotificationRequestsTotal.WithLabelValues(r.integration.Name()).Inc()
 			if err != nil {
-				r.metrics.numFailedNotifications.WithLabelValues(r.integration.Name()).Inc()
-				level.Debug(l).Log("msg", "Notify attempt failed", "attempt", i, "integration", r.integration.Name(), "receiver", r.groupName, "err", err)
+				r.metrics.numNotificationRequestsFailedTotal.WithLabelValues(r.integration.Name()).Inc()
 				if !retry {
-					return ctx, alerts, fmt.Errorf("cancelling notify retry for %q due to unrecoverable error: %s", r.integration.Name(), err)
+					return ctx, alerts, errors.Wrapf(err, "%s/%s: notify retry canceled due to unrecoverable error after %d attempts", r.groupName, r.integration.String(), i)
+				}
+				if ctx.Err() == nil && (iErr == nil || err.Error() != iErr.Error()) {
+					// Log the error if the context isn't done and the error isn't the same as before.
+					level.Warn(l).Log("msg", "Notify attempt failed, will retry later", "attempts", i, "err", err)
 				}
 
 				// Save this error to be able to return the last seen error by an
 				// integration upon context timeout.
 				iErr = err
 			} else {
+				lvl := level.Debug(l)
+				if i > 1 {
+					lvl = level.Info(l)
+				}
+				lvl.Log("msg", "Notify success", "attempts", i)
 				return ctx, alerts, nil
 			}
 		case <-ctx.Done():
-			if iErr != nil {
-				return ctx, nil, iErr
+			if iErr == nil {
+				iErr = ctx.Err()
 			}
 
-			return ctx, nil, ctx.Err()
+			return ctx, nil, errors.Wrapf(iErr, "%s/%s: notify retry canceled after %d attempts", r.groupName, r.integration.String(), i)
 		}
 	}
 }
@@ -705,18 +772,107 @@ func NewSetNotifiesStage(l NotificationLog, recv *nflogpb.Receiver) *SetNotifies
 func (n SetNotifiesStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
 	gkey, ok := GroupKey(ctx)
 	if !ok {
-		return ctx, nil, fmt.Errorf("group key missing")
+		return ctx, nil, errors.New("group key missing")
 	}
 
 	firing, ok := FiringAlerts(ctx)
 	if !ok {
-		return ctx, nil, fmt.Errorf("firing alerts missing")
+		return ctx, nil, errors.New("firing alerts missing")
 	}
 
 	resolved, ok := ResolvedAlerts(ctx)
 	if !ok {
-		return ctx, nil, fmt.Errorf("resolved alerts missing")
+		return ctx, nil, errors.New("resolved alerts missing")
 	}
 
 	return ctx, alerts, n.nflog.Log(n.recv, gkey, firing, resolved)
 }
+
+type timeStage struct {
+	Times map[string][]timeinterval.TimeInterval
+}
+
+type TimeMuteStage timeStage
+
+func NewTimeMuteStage(ti map[string][]timeinterval.TimeInterval) *TimeMuteStage {
+	return &TimeMuteStage{ti}
+}
+
+// Exec implements the stage interface for TimeMuteStage.
+// TimeMuteStage is responsible for muting alerts whose route is not in an active time.
+func (tms TimeMuteStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
+	muteTimeIntervalNames, ok := MuteTimeIntervalNames(ctx)
+	if !ok {
+		return ctx, alerts, nil
+	}
+	now, ok := Now(ctx)
+	if !ok {
+		return ctx, alerts, errors.New("missing now timestamp")
+	}
+
+	muted, err := inTimeIntervals(now, tms.Times, muteTimeIntervalNames)
+	if err != nil {
+		return ctx, alerts, err
+	}
+
+	// If the current time is inside a mute time, all alerts are removed from the pipeline.
+	if muted {
+		level.Debug(l).Log("msg", "Notifications not sent, route is within mute time")
+		return ctx, nil, nil
+	}
+	return ctx, alerts, nil
+}
+
+type TimeActiveStage timeStage
+
+func NewTimeActiveStage(ti map[string][]timeinterval.TimeInterval) *TimeActiveStage {
+	return &TimeActiveStage{ti}
+}
+
+// Exec implements the stage interface for TimeActiveStage.
+// TimeActiveStage is responsible for muting alerts whose route is not in an active time.
+func (tas TimeActiveStage) Exec(ctx context.Context, l log.Logger, alerts ...*types.Alert) (context.Context, []*types.Alert, error) {
+	activeTimeIntervalNames, ok := ActiveTimeIntervalNames(ctx)
+	if !ok {
+		return ctx, alerts, nil
+	}
+
+	// if we don't have active time intervals at all it is always active.
+	if len(activeTimeIntervalNames) == 0 {
+		return ctx, alerts, nil
+	}
+
+	now, ok := Now(ctx)
+	if !ok {
+		return ctx, alerts, errors.New("missing now timestamp")
+	}
+
+	active, err := inTimeIntervals(now, tas.Times, activeTimeIntervalNames)
+	if err != nil {
+		return ctx, alerts, err
+	}
+
+	// If the current time is not inside an active time, all alerts are removed from the pipeline
+	if !active {
+		level.Debug(l).Log("msg", "Notifications not sent, route is not within active time")
+		return ctx, nil, nil
+	}
+
+	return ctx, alerts, nil
+}
+
+// inTimeIntervals returns true if the current time is contained in one of the given time intervals.
+func inTimeIntervals(now time.Time, intervals map[string][]timeinterval.TimeInterval, intervalNames []string) (bool, error) {
+	for _, name := range intervalNames {
+		interval, ok := intervals[name]
+		if !ok {
+			return false, errors.Errorf("time interval %s doesn't exist in config", name)
+		}
+		for _, ti := range interval {
+			if ti.ContainsTime(now.UTC()) {
+				return true, nil
+			}
+		}
+	}
+	return false, nil
+}
